"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const loggerhythm_1 = require("loggerhythm");
const uuid = require("uuid");
const logger = loggerhythm_1.Logger.createLogger('pprocessengine:external_task:worker');
class ExternalTaskWorker {
    constructor(externalTaskApi) {
        this._workerId = uuid.v4();
        this._lockDuration = 30000;
        this._externalTaskApi = undefined;
        this._externalTaskApi = externalTaskApi;
    }
    get workerId() {
        return this._workerId;
    }
    async waitForAndHandle(identity, topic, maxTasks, longpollingTimeout, handleAction) {
        while (true) {
            const externalTasks = await this._fetchAndLockExternalTasks(identity, topic, maxTasks, longpollingTimeout);
            const interval = setInterval(async () => await this._extendLocks(identity, externalTasks), this._lockDuration - 5000);
            const executeTaskPromises = [];
            for (const externalTask of externalTasks) {
                executeTaskPromises.push(this._executeExternalTask(identity, externalTask, handleAction));
            }
            await Promise.all(executeTaskPromises);
            clearInterval(interval);
        }
    }
    async _fetchAndLockExternalTasks(identity, topic, maxTasks, longpollingTimeout) {
        try {
            return await this._externalTaskApi.fetchAndLockExternalTasks(identity, this._workerId, topic, maxTasks, longpollingTimeout, this._lockDuration);
        }
        catch (error) {
            logger.error(error);
            await this._sleep(1000);
            return await this._fetchAndLockExternalTasks(identity, topic, maxTasks, longpollingTimeout);
        }
    }
    async _executeExternalTask(identity, externalTask, handleAction) {
        try {
            const result = await handleAction(externalTask);
            await result.sendToExternalTaskApi(this._externalTaskApi, identity, this._workerId);
        }
        catch (error) {
            logger.error(error);
            await this._externalTaskApi.handleServiceError(identity, this._workerId, externalTask.id, error.message, '');
        }
    }
    async _extendLocks(identity, externalTasks) {
        for (const externalTask of externalTasks) {
            await this._externalTaskApi.extendLock(identity, this._workerId, externalTask.id, this._lockDuration);
        }
    }
    async _sleep(milliseconds) {
        return new Promise((resolve) => {
            setTimeout(() => resolve(), milliseconds);
        });
    }
}
exports.ExternalTaskWorker = ExternalTaskWorker;

//# sourceMappingURL=external_task_worker.js.map
